/*
 * Copyright (c) 1990, 1999 Erick Engelke
 */

typedef struct _graph_x {
    int   g_width;
    int   g_height;
    BYTE  g_background;
    BYTE *g_data;   /* assumes 8 bits */
} graph_x;

/*
 * (0,0) is bottom left, ( width-1, height-1) is upper right
 *
 */
void *gr_alloc( int width, int height );
void gr_free( void *x );
void gr_background( void *z, int col );

void gr_setpixel( void *z, int x, int y, int cols );
int gr_getpixel( void *z, int x, int y );
void gr_line( void *z, int x1, int y1, int x2, int y2, int color );


void gr_char_at( void *z , char ch, int x, int y, int color );
void gr_text_at( void *z , char *s, int x, int y, int color );
void gr_linegraph( void *z, int numpoints, int *ys,
    char *x1, char *x2, int textcolor, int color );
void gr_gif( tcp_Socket *s, void *z );

